/*
 * fuc: back-top.js v0.1.0
 */

import $ from 'jquery';
import {getRequestAnimationFrame, easeInOutCubic} from './getRequestAnimationFrame';

const reqAnimFrame = getRequestAnimationFrame();

function noop() {
}

function getScroll(target, top) {
  if (typeof window === 'undefined') {
    return 0;
  }

  const prop = top ? 'pageYOffset' : 'pageXOffset';
  const method = top ? 'scrollTop' : 'scrollLeft';
  const isWindow = target === window;

  let ret = isWindow ? target[prop] : target[method];
  // ie6,7,8 standard mode
  if (isWindow && typeof ret !== 'number') {
    ret = window.document.documentElement[method];
  }

  return ret;
}

const BackTop = (($) => {

  const NAME = 'back-top'
  const TOGGLE = `[data-toggle="${NAME}"]`
  const DATA_KEY = `fuc.${NAME}`
  const TARGET = 'data-target'
  const VHEIGHT = 'data-visibilityHeight'
  const CALLBACK = 'data-callback'

  class BackTop {

    constructor($ele) {
      this.$ele = $ele;
    }

    static VERSION = '0.1.0';

    toggle = () => {
      this.handleScroll()
    }

    show = () => {
      this.$ele.fadeIn(400)
    }

    hide = () => {
      this.$ele.fadeOut(400)
    }

    getCurrentScrollTop = () => {
      if (this.target === window) {
        return window.pageYOffset || document.body.scrollTop || document.documentElement.scrollTop;
      }
      return this.target.scrollTop;
    }

    scrollToTop = (e) => {
      const scrollTop = this.getCurrentScrollTop();
      const startTime = Date.now();
      const frameFunc = () => {
        const timestamp = Date.now();
        const time = timestamp - startTime;
        this.setScrollTop(easeInOutCubic(time, scrollTop, 0, 450));
        if (time < 450) {
          reqAnimFrame(frameFunc);
        }
      };
      reqAnimFrame(frameFunc);
      this.cb(e);
    }

    setScrollTop = (value) => {
      if (this.target === window) {
        document.body.scrollTop = value;
        document.documentElement.scrollTop = value;
      } else {
        this.target.scrollTop = value;
      }
    }

    handleScroll = () => {
      const {visibilityHeight, target} = this;
      const scrollTop = getScroll(target, true);
      if (scrollTop > visibilityHeight) {
        this.show()
      } else {
        this.hide()
      }
    }
  }


  // BackTop PLUGIN DEFINITION
  // ========================

  function Plugin(option) {
    return this.each(function () {
      const $this = $(this)
      let data = $this.data(DATA_KEY)
      let target = $this.attr(TARGET) || window
      target = typeof target === 'string' ? (document.querySelector(target) || window) : target

      if (!data) {
        data = new BackTop($this)
        data.target = target
        data.visibilityHeight = Number($this.attr(VHEIGHT)) || 400
        data.cb = window[$this.attr(CALLBACK)] || noop
        $this.data(DATA_KEY, data)
      }

      $(data.target).on('scroll', data.handleScroll)

      if (typeof option === 'string') data[option]()
    })
  }

  const old = $.fn[NAME]

  $.fn[NAME] = Plugin
  $.fn[NAME].Constructor = BackTop


  // BackTop NO CONFLICT
  // ==================

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = old
    return this
  }


  // BackTop DATA-API
  // ===============

  $(document)
    .on(`click.${DATA_KEY}.data-api`, TOGGLE, function (e) {
      const $this = $(this)
      const data = $this.data(DATA_KEY)

      if ($this.is('a')) e.preventDefault()
      data.scrollToTop(e)
    });


  $(() => {
    Plugin.call($(TOGGLE), 'toggle')
  })

})($);

export default BackTop;
